% Sub-Matlab program for :
% Gapen, Michael T. and Thomas F. Cosimano, 
% "Sovling Ramsey Problems with Nonlinear Projection Methods," 
% Studies in Nonlinear Dynamics and Econometrics, Vol. 9, Spring
% 2005.
%
% function [x,w]	= hernodes(n);
%
% Purpose: Compute the weights and nodes for Hermite-Polynomials.
% 
% Inputs: n	The number of nodes to compute
% 
% Outputs:
% x	The nodes
% w	the associated weights
%
% Source: Taken from Numerical Recipes in Fortran77, 2ed
%
% -----------------------------------------------------------------

function [x,w]	= hernodes(n);

% The relative precision.  Is used for the stopping criterion.
%
TOL	= sqrt(eps);        
%
% Maximum number of iterations
%
MAXIT 	= 30;           
PIM4	= pi^(-1/4);

% The roots are symmetric about the origin, so we only
% need to calculate half of them
%
m	= (n+1)/2;          
%
% Create the output vectors
%
x	= zeros(n,1);
w	= zeros(n,1);
%
% Loop over the desired roots
%
for i	= 1:m;

	% Initialize the first four roots
	% -------------------------------
	if i 	== 1;   % Initial guess for the largest root
		z	= sqrt(2*n+1) - 1.85575 * (2*n+1)^(-1/6);
	elseif i== 2;   % Initial guess for the second largest root
		z	= z - 1.14 * (n^.426)/z;
	elseif i== 3;   % Initial guess for the third largest root
		z	= 1.86 * z - .86 * x(1);
	elseif i== 4;   % Initial guess for the fourth largest root
		z	= 1.91 * z - .91 * x(2);
	else;           % Initial guess for the other roots
		z	= 2 * z - x(i-2);
	end;
	
    % Refinement by Newton's method
    % -----------------------------
    % Newton's method finds the polynomial p(x) that approximates f(x).
    %
	for its	= 1:MAXIT;
		p1 	= PIM4;
		p2 	= 0;
		for j	= 1:n;      % Loop up the recurrance relation to find
			p3	= p2;       % the Hermite polynomial evaluated at z
			p2	= p1;
			p1	= z*sqrt(2/j)*p2 - sqrt((j-1)/j)*p3;
		end;
        %
        % p1 is now the desired Hermite polynomial.  We next compute pp, its
        % derivative by the formula H'j = sqrt(2j)Hj-1
		%
		pp	= p2 * sqrt(2*n);
		z1	= z;
		z	= z1 - p1/pp;                   % Newton's formula
		if abs(z-z1) < TOL; break; end;     % Stopping criterion
	end;
	
	x(i)	= z;            % Store the root or node
	x(n+1-i)= -z;           % And its symmetric counterpart
	w(i)	= 2/(pp*pp);    % Compute and store the weight
	w(n+1-i)= w(i);         % And its symmetric counterpart
end;

% **********************************************************************

% **********************************************************************
