% rlquad.m
% Michael T. Gapen
%
% function real=rlquad(phi,psi);
%
% Purpose: Use Bivariate Gauss-Hermite Quadrature to evaluate 
% the real interest rate term
%
%
% =============================================================================

function real=rlquad(phi,psi);

global alpha beta a rhotheta rhog sigmatheta sigmag thetamax thetamin gmax gmin
global thetaord gord tnod gnod G0 theta0 Y g0
global C1 C2 M mu G_Y C1_Y H Kt 


% Section 1.0 Create the Gauss-Hermite Quadrature weights and nodes
% =================================================================
%
[nodtheta wtheta]=hernodes(tnod);
[nodg wg]=hernodes(gnod);
%
% Gauss-Hermite Quadrature uses the weighting function exp{-x^2}. 
% Here, however, we have the density 
% (1./sigma*sqrt(2*pi))*exp{-y^2/2*sigma^2} and want to calculate the
% integral of fun(y)*(1./sigma*sqrt(2*pi))*exp{-y^2/2*sigma^2}
%
% Adjusting the quadrature procedure requires the following two steps.
% First a change of variables is introduced. In particular, y=sqrt(2)*sigma*x.
% This results in the weighting function (1/sigma*sqrt(2*pi))*exp{-x^2}. 
% Second, we have to multiply the integrand with the Jacobian of the 
% transformation. This gives the weighting function (1/sqrt(pi))*exp{-x^2}
% and calculating the integral of fun(sqrt(2)*sigma*x)*(1/sqrt(pi))*exp{-x^2}.
%
% Furthermore, here we have a bivariate system.  The above describes a univariate
% procedure.  For a bivariate system, the weighting function is exp(-x1^2)*exp(-x2^2).
% In addition the density functions cancel terms which leaves a (pi)^(-1) instead of 
% sqrt(pi).
%
% Scale the weights by the standard deviations
%
nodestheta=sigmatheta*sqrt(2)*nodtheta;
nodesg=sigmag*sqrt(2)*nodg;
%
% Associate the weights and nodes across the state space
% 
% This loop associates the weight with each realization of the nodes.  The wrgrid is a
% (gnod*tnod) x 1 vector and nrgrid is a (gnod*tnod) x 2 matrix.  The rows of the wrgrid 
% vector give the probability of getting the nrgrid pair.
%
for ai = 1:tnod;
   for di = 1:gnod;
      wrgrid((ai-1)*gnod+di)=wtheta(ai)*wg(di);
      nrgrid((ai-1)*gnod+di,:) = [nodestheta(ai) nodesg(di)];
      end;
   end;

rdimgrid = length(nrgrid);
E1=zeros(rdimgrid,1);
%
% Section 2.0 Calculate the Expected Value of the real interest rate
% ==================================================================
% 
% Note that the polynomials below are evaluated at thetax and gx, the nodes
% of the Gauss-Hermite Quadrature over [-inf, inf].  These may be different than
% the thetaup, gup values used to create the polynomials in the remaining terms.
% The extent that thetaup and gup differ from thetax and gx will determine the amount
% of nonlinearity or curvature in the system.
%
for ri=1:rdimgrid;
       thetax=nrgrid(ri,1);
       gx=nrgrid(ri,2);
       %
       thetax1=exp(rhotheta*theta0+thetax);
       gx1=exp(rhog*g0+gx);
       %
       % No log used above since theta0=g0=0 in initial conditions and is already
       % defined as log(1)=0.
       %
       % Below, when referring to the t-1 values of theta0 and g0, these require 
       % the exponential function.
       %
       E1(ri)=(1+beta*(a/(1-a))*mfun(thetax1,gx1,psi))/(thetax1*...
           Kt*hfun(thetax1,gx1,phi)^alpha-gx1*G0);
               
   end;
%
% Note below that the weights and scaling with pi are applied only to the expectations
% term since 1 divided by the expected value of the function is not the same as
% the expected value of 1 divided by the function due to Jenson's Inequality.
% Thus, the Gauss-Hermite quadrature is applied only to the term within the expectations
% operator and not to the entire definition of the real interest rate.
%
% In addition, the real interest rate at t-1 is used, RL(t-1), and the terms below must
% account for this.  Thus, the initial conditions are used rather than the state space
% conditions since they are known at time t-1.

real=(1/beta)*(1+beta*(a/(1-a))*mfun(exp(theta0),exp(g0),psi))/...
    (exp(theta0)*Kt*(hfun(exp(theta0),exp(g0),phi))^alpha-exp(g0)*G0)*...
    (1/((1/pi)*wrgrid*E1));
